package xdoclet.maven;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.jelly.XMLOutput;
import org.apache.commons.jelly.JellyTagException;
import org.apache.commons.jelly.MissingAttributeException;

import org.apache.maven.jelly.MavenJellyContext;
import org.apache.maven.jelly.tags.BaseTagSupport;

   /**
    * Tag to iterate over all first-level extensions of a given prefix.
    * E.g. in a project there are several props defined:<br>
    * <code>
    * foo.bar.0.alfa=xxx-alfa<br>
    * foo.bar.0.beta=xxx-beta<br>
    * foo.bar.1.alfa=yyy-alfa<br>
    * foo.bar.1.beta=yyy-beta<br>
    * foo.bar.1.gamma=yyy-gamma<br>
    * </code>
    * When invoking the tag with <br>
    * &lt;ltl:propertyIterator var="theVar" prefix="foo.bar" /&gt;, the tag 
    * contents will be executed once for theVar=foo.bar.0 and once for 
    * theVar=foo.bar.1
    * 
    * @author Florin Vancea (fvancea at maxiq dot com)
    */
public class PropertyIteratorTag extends BaseTagSupport {

   /** Property: variable name */
   private String loopVarName = null;
   /** Property: prefix */
   private String prefix = null;
   /** Property: numeric */
   private boolean numeric = false;

   public void setVar(String aVar) {
      loopVarName = aVar;
   }

   public String getVar() {
      return loopVarName;
   }

   public void setPrefix(String aPrefix) {
      if (aPrefix == null) prefix = null; // will throw MissingAttributeException
      else if (!aPrefix.endsWith(".")) prefix = aPrefix + ".";
      else prefix = aPrefix;
   }

   public String getPrefix() {
      return prefix;
   }

   public void setNumeric(boolean isNumeric) {
      numeric = isNumeric;
   }

   public boolean getNumeric() {
      return numeric;
   }

   public void doTag( XMLOutput output ) 
         throws MissingAttributeException, JellyTagException {
      if (loopVarName == null || loopVarName.length() == 0)
         throw new MissingAttributeException("var");
      if (prefix == null) {
         throw new MissingAttributeException("prefix");
      }
      MavenJellyContext mavenContext = getMavenContext();
      Object savedVariable = mavenContext.getVariable(loopVarName);
      try {
         Map varMap = Util.getCascadedVars(mavenContext.getProject(), prefix);
         Iterator iter = varMap.keySet().iterator();
         Vector nameset = new Vector();
         int prefLen = prefix.length();
         while (iter.hasNext()) {
            String keyName = (String) iter.next();
            if (!keyName.startsWith(prefix)) continue;
            String propName = keyName.substring(prefLen);
            int dotIdx = propName.indexOf('.');
            if (dotIdx >= 0) {
               propName = propName.substring(0,dotIdx);
            }
            if (numeric) {
               try {
                  Integer idx = new Integer(Integer.parseInt(propName));
                  if (nameset.indexOf(idx) < 0)
                     nameset.add(idx);
               } catch(NumberFormatException nfex) {
                  continue;
               } 
            } else {
               if (nameset.indexOf(propName) < 0) 
                  nameset.add(propName);
            }
         }
         Object names[] = nameset.toArray();
         Arrays.sort(names);
         for (int i = 0; i < names.length; i++) {
            //System.out.println("Looping wih " + loopVarName + " = " + prefix + names[i].toString());
            mavenContext.setVariable(loopVarName, prefix + names[i].toString());
            invokeBody(output);
         }
      } finally {
         mavenContext.setVariable(loopVarName, savedVariable);
      }
   }

}